#include <iostream.h>
#include <fstream.h>
#include <string>
#include <ctime>	// for random numbers

#include "includes/parameters.c"
#include "includes/waveform.c"
#include "includes/transform.c"
#include "includes/hadamard.c"
#include "includes/haar.c"
#include "includes/faults.c"

#define N 16

void print_array (float * array) {
  int i;
  for (i=0; i<N; i++) {
    cout << array[i] << ", ";
  }
  cout << "\n";
}

void program_description(Parameters &p_in) {
  // prints out a description of the program
  cout << "------------------------------------\n";
  cout << " ** Waveform Format: .dgp to .dwe **\n\n";
  cout << " - takes in waveform in .dgp format\n";
  cout << " - outputs waveform in .dwe format (MCS simulator)\n";
  cout << " Requires:\n";
  cout << " - waveformfilein (old)\n";
  cout << " - waveformfileout (new)\n";
  cout << "------------------------------\n";

  // set param flag:
  p_in.set_flag(PARAM_WAVEFORMFILEIN | PARAM_WAVEFORMFILEOUT);
}

int main(int argc, char *argv[]) {

  Parameters param;
  program_description(param);

  // initialize random numbers 
  srand(static_cast<unsigned>(time(0))); 

  if (argc > 1) {
    param.load(argv[1]);
  } else {
    param.load();
  }

  Waveform * w;
  w = new Waveform;

  int init_size, final_time;
 
  init_size = w->load(param.waveformfilein);
  final_time = w->save_dwe(param.waveformfileout);

  cout << "1) input: waveform (.dgp) of length " << init_size << "\n";
  cout << "2) output: saved waveform as (.dwe) to " << param.waveformfileout;
  cout << " (final time = " << final_time << ")\n";

  delete w;
  return 0;
}
