#ifndef waveform_h
#define waveform_h

#include "waveitem.h"
#include "faults.h"

class Waveform {
 private:

  bool parse_wave(string wave);
  int wave_length;
  bool has_ommit;  // true/false if this waveform has ommit vector

  void set_ommits(bool yesno); // internal, used by ommit all/ommit none

 public:
  
  Waveitem * top;

  int num_inputs;

  // contructor/deconstructor
  Waveform();
  ~Waveform();

  // function to set the number of inputs
  // - perhaps called automatically by load
  void set_num_inputs(int num_inputs_in);
  // the current length of the waveform
  int length();
  int length_ommit();  // the length of the waveform, ommitting vectors
  // function to set if this waveform has an ommit vector or not
  // - default is false
  void ommit(bool yesno);
  // function to read the ommit value
  bool ommit();

  // returns a value
  int val(int input, int time);
  // inserts a time, returns length
  int add_time(int input);
  // adds a time to the end
  // - returns the new length
  int add_time_end();
  // changes a value (specific value on input at time)
  void change_val(int input, int time, int value);
  // chagnes a wavetime (data at time)
  void change_waveitem(int time, string new_data, bool new_ommit=false);
  // deletes a time, returns length
  int delete_time(int time);
  // pads the waveform out to a certain time, returns length
  int padd(int time);

  // wave functions
  void append_wave(Waveform *w);
  void insert_wave(Waveform *w, int time);

  // randomize all values of the wave
  void rand_wave_new(int new_wave_length, int new_num_inputs);
  void rand_wave();
  void rand_wave_time(int start_time, int rand_length);
  string rand_data();  // internal function
  // zero waveform
  void zero_wave_new(int new_wave_length, int new_num_inputs);
  void zero_wave();
  void zero_wave_time(int start_time, int zero_length);
  string zero_data(); // internal fucntion

  // fault functions:
  void ommit_all();
  void ommit_none();
  void ommits_update();  // sets all ommits to be value of future ommits
  void set_fault_ommits(Faults *f);
  void update_fault_ommits(Faults *f);
  void set_future_ommit(int time, bool yesno);

  void print ();       // prints out the waveform
  int load (string);  // loads waveform, returns length
  // loads an array into the waveform (overwriting)
  void load_array(float * arr, int arr_size, int input_num, int start_time);
  int save (string);  // saves current waveform
  int save_dwe (string); // saves current waveform as .dgp format
  // saves waveform to an array
  void save_array(float * arr, int arr_size, int input_num, int start_time);
  // save the waveform, ommitting vectors
  int save_ommit(string);
};

#endif
